import path from 'path';
import Koa from 'koa';
import serve from 'koa-static';
import Router from 'koa-router';
import serialize from 'serialize-javascript';
import { App } from './App';
import { CreateStoreProvider } from './Connect';
import { serverRender } from '@issr/core';

const app = new Koa();
const router = new Router();

app.use(serve(path.resolve(__dirname, '../public')));

router.get('/*', async (ctx) => {
  const { StoreProvider, api } = CreateStoreProvider()

  const { html } = await serverRender.string(() => (
    <StoreProvider>
      <App />
    </StoreProvider>
  ));
  console.log(api.getStore().helloWorld.state)
  ctx.body = `
  <!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>Title</title>
    <script>
      window.MOBX_DATA = ${serialize({
    helloWorld: api.getStore().helloWorld.state
  }, { isJSON: true })}
    </script>
</head>
<body>
    <div id="root">${html}</div>
    <script src="/index.js"></script>
</body>
</html>
`;
});

app
  .use(router.routes())
  .use(router.allowedMethods());

const server = app.listen(4000, () => {
  console.log(`Server is listening ${4000} port`);
});
